"""
LsVarSpoolClientmq - command ``ls -ln /var/spool/clientmqueue``
===============================================================

The ``ls -ln /var/spool/clientmqueue`` command provides information for the listing of the ``/var/spool/clientmqueue`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 40
    -rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6Wilr002718
    -rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6WixJ002715
    -rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6WjP6002721
    -rw-rw---- 1 51 51 817 Jul 11 03:35 dfw6B7Z8BB002906
    -rw-rw---- 1 51 51 817 Jul 11 04:02 dfw6B822T0011150

Examples:

    >>> "dfw6B6Wilr002718" in ls_var_spool_clientmq
    False
    >>> "/var/spool/clientmqueue" in ls_var_spool_clientmq
    True
    >>> ls_var_spool_clientmq.dir_entry('/var/spool/clientmqueue', 'dfw6B6Wilr002718')['type']
    '-'
"""

from insights.specs import Specs
from insights.util import deprecated

from .. import CommandParser, parser
from .. import FileListing


@parser(Specs.ls_var_spool_clientmq)
class LsVarSpoolClientmq(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlan` instead.

    Parses output of ``ls -ln /var/spool/clientmqueue`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarSpoolClientmq, "Please use the :class:`insights.parsers.ls.LSlan` instead.", "3.5.0")
        super(LsVarSpoolClientmq, self).__init__(*args, **kwargs)



import doctest

from insights.parsers import ls_boot
from insights.parsers.ls_boot import LsBoot
from insights.tests import context_wrap

LS_LANR_BOOT = """
/boot:
total 187380
dr-xr-xr-x.  3 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 19 0 0     4096 Jul 14 09:10 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
drwxr-xr-x.  6 0 0      111 Jun 15 09:51 grub2

/boot/grub2:
total 36
drwxr-xr-x. 6 0 0  104 Mar  4 16:16 .
dr-xr-xr-x. 3 0 0 4096 Mar  4 16:19 ..
lrwxrwxrwx. 1 0 0   11 Aug  4  2014 menu.lst -> ./grub.cfg
-rw-r--r--. 1 0 0   64 Sep 18  2015 device.map
-rw-r--r--. 1 0 0 7040 Mar 29 13:30 grub.cfg
"""

LS_ALZR_BOOT = """
/boot:
total 187380
dr-xr-xr-x.  3 root root system_u:object_r:boot_t:s0   4096 Oct  4  2023 .
dr-xr-xr-x. 19 root root system_u:object_r:boot_t:s0   4096 Apr 21  2023 ..
-rw-r--r--.  1 root root system_u:object_r:boot_t:s0 123891 Sep 30  2022 config-5.14.0-162.6.1.el9_1.aarch64
drwxr-xr-x.  6 root root system_u:object_r:boot_t:s0    111 Aug 28 16:42 grub2

/boot/grub2:
total 16
drwx------. 3 root root system_u:object_r:boot_t:s0       50 Aug 28 16:42 .
dr-xr-xr-x. 6 root root system_u:object_r:boot_t:s0     4096 Oct  4  2023 ..
drwx------. 2 root root system_u:object_r:boot_t:s0       25 Apr 21  2023 fonts
-rwx------. 1 root root system_u:object_r:boot_t:s0     6535 Apr 21  2023 grub.cfg
-rw-------. 1 root root unconfined_u:object_r:boot_t:s0 1024 Aug 28 16:42 grubenv
"""

LS_LANR_BOOT_LINKS = """
/boot:
total 187380
-rw-------.  1 root root 19143244 Mar  3 14:31 initramfs-2.6.32-504.el6.x86_64.img
lrwxrwxrwx.  1 root root       40 May 11 11:00 initrd -> initramfs-2.6.32-504.12.2.el6.x86_64.img
lrwxrwxrwx.  1 root root       34 May 11 11:00 vmlinuz -> vmlinuz-2.6.32-504.12.2.el6.x86_64
-rwxr-xr-x.  1 root root  4153904 Sep 16  2014 vmlinuz-2.6.32-504.el6.x86_64
"""

LS_ALZR_BOOT_LINKS = """
/boot:
total 187380
lrwxrwxrwx.  1 root root system_u:object_r:boot_t:s0              32 Apr 21  2023 dtb -> dtb-5.14.0-162.6.1.el9_1.aarch64
drwxr-xr-x. 11 root root system_u:object_r:boot_t:s0             129 Apr 21  2023 dtb-5.14.0-162.6.1.el9_1.aarch64
-rw-------.  1 root root system_u:object_r:boot_t:s0       103122590 Apr 21  2023 initramfs-0-rescue-6c7d8d74767a419884156df2b0005e3e.img
-rw-------.  1 root root system_u:object_r:boot_t:s0        54790153 Apr 21  2023 initramfs-5.14.0-162.6.1.el9_1.aarch64.img
-rw-------.  1 root root system_u:object_r:boot_t:s0        32324096 Oct  4  2023 initramfs-5.14.0-162.6.1.el9_1.aarch64kdump.img
drwxr-xr-x.  3 root root system_u:object_r:boot_t:s0              21 Apr 21  2023 loader
lrwxrwxrwx.  1 root root system_u:object_r:boot_t:s0              52 Apr 21  2023 symvers-5.14.0-162.6.1.el9_1.aarch64.gz -> /lib/modules/5.14.0-162.6.1.el9_1.aarch64/symvers.gz
"""


def test_ls_boot():
    ls_boot = LsBoot(context_wrap(LS_LANR_BOOT))
    assert "/boot" in ls_boot
    assert "/boot/grub2" in ls_boot
    assert ls_boot.dirs_of("/boot") == [".", "..", "grub2"]

    grub2_files = ls_boot.files_of("/boot/grub2")
    assert "menu.lst" in grub2_files
    assert "device.map" in grub2_files
    assert "grub.cfg" in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls_boot.files_of("/boot")
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    ls_alzr_boot = LsBoot(context_wrap(LS_ALZR_BOOT))
    assert "/boot" in ls_alzr_boot
    assert ls_alzr_boot.files_of("/boot") == ["config-5.14.0-162.6.1.el9_1.aarch64"]
    assert ls_alzr_boot.dirs_of("/boot") == [".", "..", "grub2"]
    assert ls_alzr_boot.get("/boot/grub2")["entries"].get("grub.cfg") == {
        "type": "-",
        "perms": "rwx------.",
        "links": 1,
        "owner": "root",
        "group": "root",
        "size": 6535,
        "se_user": "system_u",
        "se_role": "object_r",
        "se_type": "boot_t",
        "se_mls": "s0",
        "name": "grub.cfg",
        "date": "Apr 21  2023",
        "raw_entry": "-rwx------. 1 root root system_u:object_r:boot_t:s0     6535 Apr 21  2023 grub.cfg",
        "dir": "/boot/grub2",
    }


def test_boot_links():
    ls_boot = LsBoot(context_wrap(LS_LANR_BOOT_LINKS))
    boot_files = ls_boot.files_of("/boot")
    assert "/boot" in ls_boot
    assert "initramfs-2.6.32-504.el6.x86_64.img" in boot_files
    assert "initrd" in boot_files
    assert "vmlinuz" in boot_files
    assert "vmlinuz-2.6.32-504.el6.x86_64" in boot_files

    ls_alzr_boot = LsBoot(context_wrap(LS_ALZR_BOOT_LINKS))
    assert ls_alzr_boot.total_of("/boot") == 187380
    assert ls_alzr_boot.get("/boot")["entries"].get("dtb") == {
        "type": "l",
        "perms": "rwxrwxrwx.",
        "links": 1,
        "owner": "root",
        "group": "root",
        "size": 32,
        "se_user": "system_u",
        "se_role": "object_r",
        "se_type": "boot_t",
        "se_mls": "s0",
        "name": "dtb",
        "date": "Apr 21  2023",
        "link": "dtb-5.14.0-162.6.1.el9_1.aarch64",
        "raw_entry": "lrwxrwxrwx.  1 root root system_u:object_r:boot_t:s0              32 Apr 21  2023 dtb -> dtb-5.14.0-162.6.1.el9_1.aarch64",
        "dir": "/boot",
    }


def test_doc_examples():
    env = {"bootdir": LsBoot(context_wrap(LS_LANR_BOOT))}
    failed, total = doctest.testmod(ls_boot, globs=env)
    assert failed == 0



"""
NSSwitchConf - file ``/etc/nsswitch.conf``
==========================================

"""

from insights import LegacyItemAccess, Parser, parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.nsswitch_conf)
class NSSwitchConf(Parser, LegacyItemAccess):
    """
    Read the contents of the ``/etc/nsswitch.conf`` file.

    Each non-commented line is split into the service and its sources.  The
    sources (e.g. 'files sss') are stored as is, as a string.

    nsswitch.conf is case insensitive.  This means that both the service and
    its sources are converted to lower case and searches should be done
    using lower case text.

    Attributes:
        data (dict): The service dictionary
        errors (list): Non-blank lines which don't contain a ':'
        sources (set): An unordered set of the sources seen in this file

    Sample content::

        # Example:
        #passwd:    db files nisplus nis
        #shadow:    db files nisplus nis
        #group:     db files nisplus nis

        passwd:     files sss
        shadow:     files sss
        group:      files sss
        #initgroups: files

        #hosts:     db files nisplus nis dns
        hosts:      files dns myhostname

    Examples:

        >>> nss = shared[NSSwitchConf]
        >>> 'passwd' in nss
        True
        >>> 'initgroups' in nss
        False
        >>> nss['passwd']
        'files nss'
        >>> 'files' in nss['hosts']
        True
        >>> nss.errors
        []
        >>> nss.sources
        set(['files', 'dns', 'sss', 'myhostname'])

    """
    def parse_content(self, content):
        self.errors = []
        self.data = {}
        self.sources = set()
        for line in get_active_lines(content):
            if ':' not in line:
                self.errors.append(line)
            else:
                service, sources = [s.lower().strip() for s in line.split(':', 1)]
                self.data[service] = sources
                self.sources.update(set(sources.split(None)))



from glob import glob
from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from logging import getLogger
from re import findall
from sys import exit
import tempfile
from insights.util.subproc import call
import os
import six


NONCOMPLIANT_STATUS = 2
OUT_OF_MEMORY_STATUS = -9  # 247
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
SCAP_DATASTREAMS_PATH = '/usr/share/xml/scap/'
SSG_PACKAGE = 'scap-security-guide'
REQUIRED_PACKAGES = [SSG_PACKAGE, 'openscap-scanner', 'openscap']
OOM_ERROR_LINK = 'https://access.redhat.com/articles/6999111'

# SSG versions that need the <version> in XML repaired
VERSIONS_FOR_REPAIR = '0.1.18 0.1.19 0.1.21 0.1.25'.split()
SNIPPET_TO_FIX = '<version>0.9</version>'

logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, os_version=None, ssg_version=None, config=None):
        self.inventory_id = None
        self.os_major, self.os_minor = os_version if os_version else [None, None]
        self.ssg_version = ssg_version
        self.config = config
        self.conn = InsightsConnection(config)

    def download_tailoring_file(self, profile):
        if ('tailored' not in profile['attributes'] or profile['attributes']['tailored'] is False or
                ('os_minor_version' in profile['attributes'] and profile['attributes']['os_minor_version'] != self.os_minor)):
            return None

        # Download tailoring file to pass as argument to run_scan
        logger.debug(
            "Policy {0} is a tailored policy. Starting tailoring file download...".format(profile['attributes']['ref_id'])
        )
        tailoring_file_path = tempfile.mkstemp(
            prefix='oscap_tailoring_file-{0}.'.format(profile['attributes']['ref_id']),
            suffix='.xml',
            dir='/var/tmp'
        )[1]
        response = self.conn.session.get(
            "https://{0}/compliance/profiles/{1}/tailoring_file".format(self.config.base_url, profile['id'])
        )
        logger.debug("Response code: {0}".format(response.status_code))

        if not response.ok:
            logger.info("Something went wrong during downloading the tailoring file of {0}. The expected status code is 200, got {1}".format(profile['attributes']['ref_id'], response.status_code))
            return None

        if response.content is None or response.headers['Content-Type'] != "application/xml":
            logger.info("Problem with the content of the downloaded tailoring file of {0}. The expected format is xml, got {1}".format(profile['attributes']['ref_id'], response.headers['Content-Type']))
            return None

        with open(tailoring_file_path, mode="w+b") as f:
            f.write(response.content)
            logger.info("Saved tailoring file for {0} to {1}".format(profile['attributes']['ref_id'], tailoring_file_path))

        logger.debug("Policy {0} tailoring file download finished".format(profile['attributes']['ref_id']))

        return tailoring_file_path

    def get_profiles(self, search):
        response = self.conn.session.get("https://{0}/compliance/profiles".format(self.config.base_url),
                                         params={'search': search, 'relationships': 'false'})
        logger.debug("Content of the response: {0} - {1}".format(response,
                                                                 response.content))
        if response.status_code == 200:
            return response.json().get('data', [])
        else:
            return []

    def get_initial_profiles(self):
        inventory_id = self._get_inventory_id()
        return self.get_profiles('system_ids={0} canonical=false external=false'.format(inventory_id))

    def get_profiles_matching_os(self):
        inventory_id = self._get_inventory_id()
        return self.get_profiles('system_ids={0} canonical=false os_minor_version={1}'.format(inventory_id, self.os_minor))

    def profile_union_by_ref_id(self, prioritized_profiles, merged_profiles):
        profiles = dict((p['attributes']['ref_id'], p) for p in merged_profiles)
        profiles.update(dict((p['attributes']['ref_id'], p) for p in prioritized_profiles))

        profiles = list(profiles.values())
        if not profiles:
            logger.error("System is not associated with any profiles. Assign profiles using the Compliance web UI.\n")
            exit(constants.sig_kill_bad)

        return profiles

    def profile_files(self):
        return glob("{0}*rhel{1}-ds.xml".format(POLICY_FILE_LOCATION, self.os_major))

    def find_scap_policy(self, profile_ref_id):
        grepcmd = 'grep -H ' + profile_ref_id + ' ' + ' '.join(self.profile_files())
        if not six.PY3:
            grepcmd = grepcmd.encode()
        rc, grep = call(grepcmd, keep_rc=True)
        if rc:
            logger.error('XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep))
            return None
        filenames = findall(SCAP_DATASTREAMS_PATH + '.+xml', grep)
        if not filenames:
            logger.error('No XML profile files found matching ref_id {0}\n{1}\n'.format(profile_ref_id, ' '.join(filenames)))
            exit(constants.sig_kill_bad)
        return filenames[0]

    def build_oscap_command(self, profile_ref_id, policy_xml, output_path, tailoring_file_path):
        command = 'oscap xccdf eval --profile ' + profile_ref_id
        if tailoring_file_path:
            command += ' --tailoring-file ' + tailoring_file_path
        command += ' --results ' + output_path + ' ' + policy_xml
        return command

    def run_scan(self, profile_ref_id, policy_xml, output_path, tailoring_file_path=None):
        if policy_xml is None:
            return
        logger.info('Running scan for {0}... this may take a while'.format(profile_ref_id))
        env = os.environ.copy()
        env.update({'TZ': 'UTC'})
        oscap_command = self.build_oscap_command(profile_ref_id, policy_xml, output_path, tailoring_file_path)
        if not six.PY3:
            oscap_command = oscap_command.encode()
        rc, oscap = call(oscap_command, keep_rc=True, env=env)

        if rc and rc == OUT_OF_MEMORY_STATUS:
            logger.error('Scan failed due to insufficient memory')
            logger.error('More information can be found here: {0}'.format(OOM_ERROR_LINK))
            exit(constants.sig_kill_bad)

        if rc and rc != NONCOMPLIANT_STATUS:
            logger.error('Scan failed')
            logger.error(rc)
            logger.error(oscap)
            exit(constants.sig_kill_bad)

    # Helper function that traverses through the XCCDF report and replaces the content of each
    # matching xpath with an empty string
    def obfuscate(self, tree, xpaths):
        for xpath in xpaths:
            for node in tree.findall(xpath):
                node.text = ''

    def results_need_repair(self):
        return self.ssg_version in VERSIONS_FOR_REPAIR

    def repair_results(self, results_file, content):
        if not self.ssg_version:
            logger.warning("Couldn't repair SSG version in results file %s", results_file)
            return

        replacement = '<version>{0}</version>'.format(self.ssg_version)
        is_repaired = False
        for idx, line in enumerate(content):
            if not (is_repaired or SNIPPET_TO_FIX not in line):
                content[idx] = line.replace(SNIPPET_TO_FIX, replacement)
                is_repaired = True
                logger.debug(
                    'Substituted "%s" with "%s" in %s',
                    SNIPPET_TO_FIX, replacement, results_file
                )

        if is_repaired:
            logger.debug('Repaired version in results file %s', results_file)

        return content

    def _get_inventory_id(self):
        if self.inventory_id:
            return self.inventory_id

        systems = self.conn._fetch_system_by_machine_id()
        if type(systems) is list and len(systems) == 1 and 'id' in systems[0]:
            self.inventory_id = systems[0].get('id')
            return self.inventory_id
        else:
            logger.error('Failed to find system in Inventory')
            exit(constants.sig_kill_bad)



from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.ovs_vsctl_show)
class OVSvsctlshow(CommandParser):
    """
    Input example:
    ====================
    aa3f16d3-9534-4b74-8e9e-8b0ce94038c5
    Bridge br-ctlplane
        Port "eth0"
            Interface "eth0"
        Port br-ctlplane
            Interface br-ctlplane
                type: internal
        Port phy-br-ctlplane
            Interface phy-br-ctlplane
                type: patch
                options: {peer=int-br-ctlplane}
    Bridge br-int
        fail_mode: secure
        Port br-int
            Interface br-int
                type: internal
        Port int-br-ctlplane
            Interface int-br-ctlplane
            type: patch
            options: {peer=phy-br-ctlplane}
        Port "tap293d29ec-d1"
            tag: 1
            Interface "tap293d29ec-d1"
                type: internal
    ovs_version: "2.3.2"

   Parse result:
   ===============================
    {
        "uuid": "aa3f16d3-9534-4b74-8e9e-8b0ce94038c5"
        "bridges": {
            "br-ctlplane": {
                "ports": [
                    {
                        "interface": "eth0",
                        "port": "eth0"
                    },
                    {
                        "interface": "br-ctlplane",
                        "port": "br-ctlplane",
                        "type": "internal"
                    },
                    {
                        "interface": "phy-br-ctlplane",
                        "options": {
                            "peer": "int-br-ctlplane"
                        },
                        "port": "phy-br-ctlplane",
                        "type": "patch"
                    }
                ]
            },
            "br-int": {
                "fail_mode": "secure",
                "ports": [
                    {
                        "interface": "br-int",
                        "port": "br-int",
                        "type": "internal"
                    },
                    {
                        "interface": "int-br-ctlplane",
                        "options": {
                            "peer": "phy-br-ctlplane"
                        },
                        "port": "int-br-ctlplane",
                        "type": "patch"
                    },
                    {
                        "interface": "tap293d29ec-d1",
                        "port": "tap293d29ec-d1",
                        "tag": "1",
                        "type": "internal"
                    }
                ]
            }
        },
        "ovs_version": "2.3.2",
    }
    """

    def parse_content(self, content):
        if len(content) < 3:
            return
        version_line = content[-1]
        self.data = {"uuid": content[0], "bridges": {}, "ovs_version": version_line.split('"')[1]}
        bridge_dict = {}
        port_dict = {}
        for line in content[1:-1]:
            line = line.strip()
            prefix, _, value = line.partition(" ")
            if prefix == "Bridge":
                bridge_dict = {"ports": []}
                self.data["bridges"][value] = bridge_dict
            elif prefix == "fail_mode:":
                bridge_dict["fail_mode"] = value
            elif prefix == "Port":
                port_dict = {"port": value.replace('"', "")}
                bridge_dict["ports"].append(port_dict)
            elif prefix == "tag:":
                port_dict["tag"] = value
            elif prefix == "Interface":
                port_dict["interface"] = value.replace('"', "")
            elif prefix == "type:":
                port_dict["type"] = value
            elif prefix == "options:":
                options_items = value[1:-1].split(", ")
                port_dict["options"] = dict(
                    (item.split("=")[0], item.split("=")[1].replace('"', ""))
                    for item in options_items
                )

    def get_ovs_version(self):
        return self.data.get("ovs_version")

    def get_bridge(self, bridge_name):
        return self.data["bridges"].get(bridge_name, None)



"""
IsRhel - components to check RHEL version
=========================================

``IsRhel#``,  ``IsGtRhel#``, ``IsGtOrRhel#`` component are included in this
module.

The component is valid if the RHEL version got from the
:py:class:`insights.combiners.redhat_release.RedHatRelease` of the current
host satisfies the requirements.  Otherwise, it raises a
:py:class:`insights.core.exceptions.SkipComponent` to prevent dependent
components from triggering.

In particular, the component can be added as a dependency of another
components, e.g. Specs or Parsers parser to limit it to a given version.
"""
from insights.combiners.redhat_release import RedHatRelease
from insights.core.exceptions import SkipComponent
from insights.core.plugins import component


class IsRhel(object):
    """
    This component uses ``RedHatRelease`` combiner to determine the RHEL
    major version. It then checks if the major version matches the version
    argument, if it doesn't it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL.

    Raises:
        SkipComponent: When RHEL major version does not match version.
    """
    def __init__(self, rhel, major=None):
        if rhel.major != major:
            raise SkipComponent("Not RHEL {0}".format(major))
        self.minor = rhel.minor


class IsGtRhel(object):
    """
    This component uses ``RedHatRelease`` combiner to determine the RHEL
    version. It then checks if the major version matches the version argument,
    if it doesn't it raises ``SkipComponent``.

    When `equal` is set to True, it checks "greater than or equal".  Otherwise,
    it only checks "greater than".

    Attributes:
        major (int): The major version of RHEL.
        minor (int): The minor version of RHEL.

    Raises:
        SkipComponent: When RHEL version does not match the specified version.
    """
    def __init__(self, rhel, major, minor, equal=False):
        if rhel.major < major:
            raise SkipComponent("Not RHEL newer than {0}.{1}".format(major, minor))
        if equal:
            if rhel.major == major and rhel.minor < minor:
                raise SkipComponent("Not RHEL newer than or equal {0}.{1}".format(major, minor))
        else:
            if rhel.major == major and rhel.minor <= minor:
                raise SkipComponent("Not RHEL newer than {0}.{1}".format(major, minor))
        self.major = rhel.major
        self.minor = rhel.minor


@component(RedHatRelease)
class IsRhel6(IsRhel):
    """
    This component checks if it's RHEL 6.

    Attributes:
        minor (int): The minor version of RHEL 6.

    Raises:
        SkipComponent: When RHEL version is not RHEL 6.
    """
    def __init__(self, rhel):
        super(IsRhel6, self).__init__(rhel, 6)


@component(RedHatRelease)
class IsRhel7(IsRhel):
    """
    This component checks if it's RHEL 7.

    Attributes:
        minor (int): The minor version of RHEL 7.

    Raises:
        SkipComponent: When RHEL version is not RHEL 7.
    """
    def __init__(self, rhel):
        super(IsRhel7, self).__init__(rhel, 7)


@component(RedHatRelease)
class IsRhel8(IsRhel):
    """
    This component checks if it's RHEL 8.

    Attributes:
        minor (int): The minor version of RHEL 8.

    Raises:
        SkipComponent: When RHEL version is not RHEL 8.
    """
    def __init__(self, rhel):
        super(IsRhel8, self).__init__(rhel, 8)


@component(RedHatRelease)
class IsRhel9(IsRhel):
    """
    This component checks if it's RHEL 9.

    Attributes:
        minor (int): The minor version of RHEL 9.

    Raises:
        SkipComponent: When RHEL version is not RHEL 9.
    """
    def __init__(self, rhel):
        super(IsRhel9, self).__init__(rhel, 9)


@component(RedHatRelease)
class IsGtOrRhel86(IsGtRhel):
    """
    This component checks if the RHEL version is 8.6 or grater than 8.6.

    Attributes:
        major (int): The major version of RHEL.
        minor (int): The minor version of RHEL.

    Raises:
        SkipComponent: When RHEL version is not 8.6 and less than 8.6
    """
    def __init__(self, rhel):
        super(IsGtOrRhel86, self).__init__(rhel, 8, 6, equal=True)


@component(RedHatRelease)
class IsGtRhel86(IsGtRhel):
    """
    This component checks if the RHEL version is grater than 8.6.

    Attributes:
        major (int): The major version of RHEL.
        minor (int): The minor version of RHEL.

    Raises:
        SkipComponent: When RHEL version is 8.6 or less than 8.6
    """
    def __init__(self, rhel):
        super(IsGtRhel86, self).__init__(rhel, 8, 6, equal=False)



from insights.tests import context_wrap
from insights.parsers.x86_debug import X86PTIEnabled, X86IBPBEnabled, X86IBRSEnabled, X86RETPEnabled
from insights.combiners import x86_page_branch
from insights.combiners.x86_page_branch import X86PageBranch
import doctest


def test_values():
    for val in ['0', '1', '2']:
        ptienabled = X86PTIEnabled(context_wrap(val))
        ibpbenabled = X86IBPBEnabled(context_wrap(val))
        ibrsenabled = X86IBRSEnabled(context_wrap(val))
        res = X86PageBranch(ptienabled, ibpbenabled, ibrsenabled, None)
        assert res.pti == int(val)
        assert res.ibpb == int(val)
        assert res.ibrs == int(val)
        assert res.retp is None

    for val in ['0', '1', '2']:
        ptienabled = X86PTIEnabled(context_wrap(val))
        ibpbenabled = X86IBPBEnabled(context_wrap(val))
        ibrsenabled = X86IBRSEnabled(context_wrap(val))
        retpenabled = X86RETPEnabled(context_wrap(val))
        res = X86PageBranch(ptienabled, ibpbenabled, ibrsenabled, retpenabled)
        assert res.pti == int(val)
        assert res.ibpb == int(val)
        assert res.ibrs == int(val)
        assert res.retp == int(val)


def test_x86_enabled_documentation():
    """
    Here we test the examples in the documentation automatically using
    doctest.  We set up an environment which is similar to what a rule
    writer might see - a '/sys/kernel/debug/x86/*_enabled' output
    that has been passed in as a parameter to the rule declaration.
    """
    env = {
            'dv': X86PageBranch(X86PTIEnabled(context_wrap('1')),
                X86IBPBEnabled(context_wrap('3')),
                X86IBRSEnabled(context_wrap('2')),
                X86RETPEnabled(context_wrap('0')))
                }
    failed, total = doctest.testmod(x86_page_branch, globs=env)
    assert failed == 0



import os
from insights.core import dr
from insights.core import Parser
from insights.core.exceptions import ParseException, ContentException
from insights.core.plugins import datasource, parser, rule, make_info
from insights.core.spec_factory import (
        DatasourceProvider, RegistryPoint, SpecSet, TextFileProvider)

from insights.tests import InputData
from insights.tests import run_input_data

EXPECTED_MSG_1 = 'Invalid Data 1'
EXPECTED_MSG_2 = 'Invalid Data 2'
EXPECTED_MSG_3 = 'Invalid Data 3'


class Specs(SpecSet):
    the_ce_data = RegistryPoint()
    the_ex_data = RegistryPoint()
    the_ds_data = RegistryPoint()
    the_ft_data = RegistryPoint(filterable=True)


@datasource()
def SomeData(broker):
    return DatasourceProvider("invalid data", "dummy")


@datasource()
def FilterData(broker):
    return TextFileProvider(os.path.abspath(__file__).rstrip("c"),
            ds=Specs.the_ft_data)


class TestSpecs(Specs):
    @datasource()
    def the_ce_data(broker):
        raise ContentException(EXPECTED_MSG_1)

    @datasource()
    def the_ex_data(broker):
        raise Exception(EXPECTED_MSG_2)
    the_ds_data = SomeData
    the_ft_data = FilterData


@parser(Specs.the_ds_data)
class SomeParser(Parser):
    def parse_content(self, content):
        raise ParseException(EXPECTED_MSG_3)


@parser(Specs.the_ft_data)
class FilterSpecParser(Parser):
    def parse_content(self, content):
        return content


@rule(Specs.the_ce_data, Specs.the_ex_data)
def report(ce, ex):
    return make_info('INFO_1')

#
# TEST
#


def test_broker_spec_exceptions():
    broker = dr.run(report)
    assert report in broker
    assert Specs.the_ce_data in broker.exceptions
    spec_exs = broker.exceptions[Specs.the_ce_data]
    exs = [ex for ex in spec_exs if isinstance(ex, ContentException)]
    assert len(exs) == 1
    tb = broker.tracebacks[exs[0]]
    assert type(tb) is str
    assert "Traceback" in tb
    assert EXPECTED_MSG_1 in tb

    assert Specs.the_ex_data in broker.exceptions
    spec_exs = broker.exceptions[Specs.the_ex_data]
    exs = [ex for ex in spec_exs if isinstance(ex, Exception)]
    assert len(exs) == 1
    tb = broker.tracebacks[exs[0]]
    assert type(tb) is str
    assert "Traceback" in tb
    assert EXPECTED_MSG_2 in tb


def test_broker_parse_exception():
    broker = run_input_data(SomeParser, InputData())
    assert SomeParser in broker.exceptions
    exs = broker.exceptions[SomeParser]
    exs = [ex for ex in exs if isinstance(ex, ParseException)]
    assert len(exs) == 1
    tb = broker.tracebacks[exs[0]]
    assert type(tb) is str
    assert "Traceback" in tb
    assert EXPECTED_MSG_3 in tb


def test_no_filter_exception():
    # No "add_filter" to Specs.the_ft_data or FilterSpecParser
    broker = run_input_data(FilterSpecParser, InputData())
    # Parser can be run, when no 'add_filter" for 'filterable' specs
    assert Specs.the_ft_data in broker
    assert 'def test_no_filter_exception():' in broker[Specs.the_ft_data].content
    # and NO "no filter exception"
    exs = broker.exceptions[Specs.the_ft_data]
    assert len(exs) == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import abrt_ccpp
from insights.parsers.abrt_ccpp import AbrtCCppConf
from insights.tests import context_wrap

ABRT_CONF_CONTENT = """
# Configuration file for CCpp hook

# CCpp hook writes its template to the "/proc/sys/kernel/core_pattern" file
# and stores the original template in the "/var/run/abrt/saved_core_pattern"
# file. If you want CCpp hook to create a core dump file named according to
# the original template as well, set 'MakeCompatCore' to 'yes'.
# If the original template string starts with "|", the string "core" is used
# instead of the template.
# For more information about naming core dump files see 'man 5 core'.
MakeCompatCore = yes

# The option allows you to set limit for the core file size in MiB.
#
# This value is compared to value of the MaxCrashReportSize configuration
# option from (/etc/abrt.conf) and the lower value is used as the limit.
#
# If MaxCoreFileSize is 0 then the value of MaxCrashReportSize is the limit.
# If MaxCrashReportSize is 0 then the value of MaxCoreFileSize is the limit.
# If both values are 0 then the core file size is unlimited.
MaxCoreFileSize = 0

# Do you want a copy of crashed binary be saved?
# (useful, for example, when _deleted binary_ segfaults)
SaveBinaryImage = no

# When this option is set to 'yes', core backtrace is generated
# from the memory image of the crashing process. Only the crash
# thread is present in the backtrace.
CreateCoreBacktrace = yes

# Save full coredump? If set to 'no', coredump won't be saved
# and you won't be able to report the crash to Bugzilla. Only
# useful with CreateCoreBacktrace set to 'yes'. Please
# note that if this option is set to 'no' and MakeCompatCore
# is set to 'yes', the core is still written to the current
# directory.
SaveFullCore = yes

# Used for debugging the hook
#VerboseLog = 2

# Specify where you want to store debuginfos (default: /var/cache/abrt-di)
#
DebuginfoLocation = /var/cache/abrt-di

# ABRT will ignore crashes in executables whose absolute path matches one of
# specified patterns.
#
#IgnoredPaths =

# ABRT will process only crashes of either allowed users or users who are
# members of allowed group. If no allowed users nor allowed group are specified
# ABRT will process crashes of all users.
#
#AllowedUsers =
#AllowedGroups =
""".strip()

ABRT_CONF_CONTENT_NO = """
""".strip()


def test_empty_content():
    with pytest.raises(SkipComponent):
        AbrtCCppConf(context_wrap(ABRT_CONF_CONTENT_NO))


def test_abrt_class():
    abrt_obj = AbrtCCppConf(context_wrap(ABRT_CONF_CONTENT))
    assert abrt_obj.get('CreateCoreBacktrace', '').lower() == 'yes'
    assert abrt_obj.get('DebuginfoLocation', '').lower() == '/var/cache/abrt-di'
    assert abrt_obj.get('Debuginfo', '').lower() == ''


def test_docs():
    env = {
        'abrt_conf': AbrtCCppConf(context_wrap(ABRT_CONF_CONTENT))
    }
    failed, total = doctest.testmod(abrt_ccpp, globs=env)
    assert failed == 0



"""
CUPS configuration files
========================

Parsers provided by this module are:

CupsdConf - file ``/etc/cups/cupsd.conf``
-----------------------------------------
CupsBrowsedConf - file ``/etc/cups/cups-browsed.conf``
------------------------------------------------------
CupsFilesConf - file ``/etc/cups/cups-files.conf``
--------------------------------------------------
"""

from insights.core import Parser, ConfigParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.parsers import get_active_lines
from insights.parsers.httpd_conf import DocParser
from insights.core.exceptions import ParseException
from insights.parsr.query import Entry


class CupsdConfBase(DocParser):

    def __call__(self, content):
        try:
            return self.Top(content)
        except Exception:
            raise ParseException("There was an exception when parsing the cupsd.conf file")


@parser(Specs.cupsd_conf)
class CupsdConf(ConfigParser):
    """
    Class for parsing the file ``/etc/cups/cupsd.conf``

    Sample file content::

        MaxLogSize 0
        LogLevel warn
        Listen localhost:631
        Listen /var/run/cups/cups.sock
        Browsing On
        BrowseLocalProtocols dnssd

        <Location />
          Order allow,deny
        </Location>

        <Policy default>
          JobPrivateAccess default
          JobPrivateValues default
          SubscriptionPrivateAccess default
          SubscriptionPrivateValues default

          <Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job Cancel-My-Jobs Close-Job CUPS-Move-Job CUPS-Get-Document>
            Require user @OWNER @SYSTEM
            Order deny,allow
          </Limit>

          <Limit All>
            Order deny,allow
          </Limit>
        </Policy>

    Examples:
        >>> type(cupsd_conf)
        <class 'insights.parsers.cups_confs.CupsdConf'>
        >>> cupsd_conf['MaxLogSize'][-1].value
        0
        >>> cupsd_conf['Listen'][0].value
        'localhost:631'
        >>> cupsd_conf['Listen'][-1].value
        '/var/run/cups/cups.sock'
        >>> cupsd_conf[('Location', '/')]['Order'][-1].value
        'allow,deny'
        >>> cupsd_conf[('Policy','default')][('Limit','Send-Document')]['Order'][0].value
        'deny,allow'
        >>> 'JobPrivateAccess' in cupsd_conf[('Policy','default')]
        True
    """

    def __init__(self, *args, **kwargs):
        self.parse = CupsdConfBase(self)
        super(CupsdConf, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        if isinstance(content, list):
            content = "\n".join(content)
        result = self.parse(content)[0]
        return Entry(children=result, src=self)


@parser(Specs.cups_browsed_conf)
class CupsBrowsedConf(Parser, dict):
    """
    Class for parsing the file ``/etc/cups/cups-browsed.conf``

    .. note::

        The admin can add multiple directives into the configuration file, and restart service without issue.
        However, item like BrowseRemoteProtocols only works for the last one, and the item like BrowseAllow works
        for all directives. So the values of each directives will get stored to a list with the original order,
        and duplicated values will be kept without de-duplication.

    Sample file content::

        BrowseRemoteProtocols dnssd cups
        BrowseAllow cups.example.com

    Examples:
        >>> type(cups_browsed_conf)
        <class 'insights.parsers.cups_confs.CupsBrowsedConf'>
        >>> 'dnssd cups' in cups_browsed_conf['BrowseRemoteProtocols']
        True
        >>> 'cups.example.com' in cups_browsed_conf['BrowseAllow']
        True
    """
    def parse_content(self, content):
        if not content:
            raise ParseException('Empty Content')

        for line in get_active_lines(content):
            k, v = [i.strip() for i in line.split(None, 1)]
            if k not in self:
                self[k] = [v]
            else:
                self[k].append(v)


@parser(Specs.cups_files_conf)
class CupsFilesConf(Parser, dict):
    """
    Class for parsing the file ``/etc/cups/cups-files.conf``

    Sample file content::

        FatalErrors config
        SyncOnClose Yes
        SystemGroup sys root wheel
        AccessLog syslog
        ErrorLog syslog
        PageLog syslog

    Examples:
        >>> type(cups_files_conf)
        <class 'insights.parsers.cups_confs.CupsFilesConf'>
        >>> cups_files_conf['FatalErrors']
        'config'
        >>> cups_files_conf['SyncOnClose']
        'Yes'
        >>> 'wheel' in cups_files_conf['SystemGroup']
        True
    """

    def parse_content(self, content):
        if not content:
            raise ParseException('Empty Content')

        for line in get_active_lines(content):
            k, v = [i.strip() for i in line.split(None, 1)]
            if k not in self:
                self[k] = v if len(v.split()) == 1 else v.split()
            else:
                _v = self[k]
                _v = [_v] if not isinstance(_v, list) else _v
                if v not in _v:
                    _v.append(v)
                    self[k] = _v



"""
meminfo - file ``/proc/meminfo``
================================

This suite of parsers deals with various parts of the contents of
``/proc/meminfo``.  They store the data for many different groupings of
memory usage information as key-value pairs and attributes.  Key strings are
converted to lower case, and all values are stored as integers.  Data stored
in kilobytes (i.e. everything but the hugepage values) are converted to bytes
by multiplying by 1024.

All keys are stored in the parser class as properties.  The information
relevant to particular uses of memory are also available in the following
properties:

``swap``: for swap related information:

* ``total`` - the *SwapTotal* information
* ``free`` - the *SwapFree* information
* ``cached`` - the *SwapCached* information
* ``used`` - total - (free + cached)

``anon``: for anonymous page information:

* ``active`` - the *Active(anon)* information
* ``inactive`` - the *Inactive(anon)* information
* ``pages`` - the *AnonPages* information

``file``: for file mapping information:

* ``active`` - the *Active(file)* information
* ``inactive`` - the *Inactive(file)* information

``slab``: for SLAB allocator information:

* ``total`` - the *Slab* information
* ``reclaimable`` - the *SReclaimable* information
* ``unreclaimable`` - the *SUnreclaim* information

``huge_pages``: for HugePage allocator information:

* ``total`` - the *Hugepages_Total* information
* ``free`` - the *Hugepages_Free* information
* ``reserved`` - the *Hugepages_Rsvd* information
* ``surplus`` - the *Hugepages_Surp* information
* ``size`` - the *HugepageSize* information
* ``anon`` - the *AnonHugepages* information

``huge_pages`` also contains two properties to help parsers determine whether
huge pages are in use:

* ``using`` - are huge pages in use?  Is *Hugepages_Total* > 0?
* ``using_transparent`` - are transparent huge pages in use?  Is
  *AnonHugePages* > 0?


``commit``: for memory overcommit information:

* ``total`` - the *Committed_As* information
* ``limit`` - the *CommitLimit* information

``vmalloc``: for virtual memory allocation information:

* ``total`` - the *VMAllocTotal* information
* ``used`` - the *VMAllocUsed* information
* ``chunk`` - the *VMAllocChunk* information

``cma``: the CMA information:

* ``total`` - the *CMAllocTotal* information
* ``free`` - the *CMAllocFree* information

``direct_map``: the direct memory map information

* ``kb`` - the *DirectMap4K* information
* ``mb`` - the *DirectMap2M* information
* ``gb`` - the *DirectMap1G* information

Sample data::

    MemTotal:        8009912 kB
    MemFree:          538760 kB
    MemAvailable:    6820236 kB
    Buffers:          157048 kB
    Cached:          4893932 kB
    SwapCached:          120 kB
    Active:          2841500 kB
    Inactive:        2565560 kB
    Active(anon):     311596 kB
    Inactive(anon):   505800 kB
    Active(file):    2529904 kB
    Inactive(file):  2059760 kB
    ...

Examples:

    >>> mem = shared[MemInfo]
    >>> m.data.['memtotal'] # Old style accessor
    8202149888
    >>> mem.total # New property-based accessor
    8202149888
    >>> mem.used # Calculated
    7650459648
    >>> m.swap.total
    3221221376
    >>> m.swap.free
    3211624448
    >>> m.swap.used # Calculated
    9474048
"""

from .. import Parser, parser, get_active_lines
from insights.specs import Specs


class SubMemInfo(object):

    sub_keys = []

    def __init__(self, data):
        sub_dict = dict((sk, data.get(k)) for k, sk in self.sub_keys)
        self.data = sub_dict
        for k, v in sub_dict.items():
            setattr(self, k, v)


class SwapMemInfo(SubMemInfo):

    sub_keys = [
        ("swaptotal", "total"),
        ("swapfree", "free"),
        ("swapcached", "cached")
    ]

    @property
    def used(self):
        return self.total - self.free - self.cached


class AnonMemInfo(SubMemInfo):

    sub_keys = [
        ("active(anon)", "active"),
        ("inactive(anon)", "inactive"),
        ("anonpages", "pages")
    ]


class FileMemInfo(SubMemInfo):

    sub_keys = [
        ("active(file)", "active"),
        ("inactive(file)", "inactive")
    ]


class SlabMemInfo(SubMemInfo):

    sub_keys = [
        ("slab", "total"),
        ("sreclaimable", "reclaimable"),
        ("sunreclaim", "unreclaimable")
    ]


class HugePageMemInfo(SubMemInfo):

    sub_keys = [
        ("hugepages_total", "total"),
        ("hugepages_free", "free"),
        ("hugepages_rsvd", "reserved"),
        ("hugepages_surp", "surplus"),
        ("hugepagesize", "size"),
        ("anonhugepages", "anon")
    ]

    @property
    def using(self):
        return bool(self.total)

    @property
    def using_transparent(self):
        return bool(self.anon)


class CommitMemInfo(SubMemInfo):

    sub_keys = [
        ("committed_as", "total"),
        ("commitlimit", "limit")
    ]


class VmallocMemInfo(SubMemInfo):

    sub_keys = [
        ("vmalloctotal", "total"),
        ("vmallocused", "used"),
        ("vmallocchunk", "chunk")
    ]


class CmaMemInfo(SubMemInfo):

    sub_keys = [
        ("cmatotal", "total"),
        ("cmafree", "free")
    ]


class DirectMapMemInfo(SubMemInfo):

    sub_keys = [
        ("directmap4k", "kb"),
        ("directmap2m", "mb"),
        ("directmap1g", "gb")
    ]


@parser(Specs.meminfo)
class MemInfo(Parser):
    """
    Meminfo field names are wildly inconsistent (imho).  This class attempts to
    bring a bit of order to the chaos.  All values are in bytes.

    KB describing /proc/meminfo

    https://access.redhat.com/solutions/406773
    """

    VALUE_IN_BYTES = [
        "hugepages_total", "hugepages_free", "hugepages_rsvd", "hugepages_surp"
    ]

    mem_keys = [
        ("memtotal", "total"),
        ("memfree", "free"),
        ("memavailable", "available"),
        ("buffers", "buffers"),
        ("cached", "cached"),
        ("active", "active"),
        ("inactive", "inactive"),
        ("unevictable", "unevictable"),
        ("dirty", "dirty"),
        ("writeback", "writeback"),
        ("mapped", "mapped"),
        ("shmem", "shmem"),
        ("mlocked", "mlocked"),
        ("kernelstack", "kernel_stack"),
        ("pagetables", "page_tables"),
        ("nfs_unstable", "nfs_unstable"),
        ("bounce", "bounce"),
        ("writebacktmp", "writeback_tmp"),
        ("hardwarecorrupted", "corrupted"),
    ]

    def __init__(self, context):
        super(MemInfo, self).__init__(context)
        sub_classes = {
            "anon": AnonMemInfo(self.data),
            "file": FileMemInfo(self.data),
            "swap": SwapMemInfo(self.data),
            "slab": SlabMemInfo(self.data),
            "huge_pages": HugePageMemInfo(self.data),
            "commit": CommitMemInfo(self.data),
            "vmalloc": VmallocMemInfo(self.data),
            "cma": CmaMemInfo(self.data),
            "direct_map": DirectMapMemInfo(self.data)
        }
        for name, cls in sub_classes.items():
            setattr(self, name, cls)
        for meminfo_key, k in self.mem_keys:
            setattr(self, k, self.data.get(meminfo_key))

    def parse_content(self, content):
        self.data = {}
        for line in get_active_lines(content, comment_char="COMMAND>"):
            (key, value) = line.split(':', 1)
            # Store values as byte count
            key = key.strip().lower()
            if key in MemInfo.VALUE_IN_BYTES:
                self.data[key.strip().lower()] = int(value.split()[0])
            else:
                self.data[key.strip().lower()] = int(value.split()[0]) * 1024

    @property
    def used(self):
        return self.total - self.free



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import rhui_release
from insights.parsers.rhui_release import RHUISetRelease, RHUIReleaseVer
from insights.tests import context_wrap

INPUT_NORMAL_1 = """
8.6
""".strip()

INPUT_NORMAL_2 = """
7Server
""".strip()

INPUT_NORMAL_3 = """
8
""".strip()

INPUT_NOT_SET = """""".strip()

INPUT_NG_1 = """
XYC
Release not set
""".strip()

INPUT_NG_2 = """
abc def
""".strip()

INPUT_NG_3 = """
ab.def
""".strip()


def test_rhui_release_ok():
    ret = RHUISetRelease(context_wrap(INPUT_NORMAL_1))
    assert ret.set == '8.6'
    assert ret.major == 8
    assert ret.minor == 6


def test_rhui_release_not_set():
    ret = RHUISetRelease(context_wrap(INPUT_NOT_SET))
    assert ret.set is None
    assert ret.major is None
    assert ret.minor is None


def test_rhui_release_show_ng():
    with pytest.raises(SkipComponent) as e_info:
        RHUISetRelease(context_wrap(INPUT_NG_1))
    assert "Unexpected content" in str(e_info.value)

    with pytest.raises(SkipComponent) as e_info:
        RHUISetRelease(context_wrap(INPUT_NG_2))
    assert "Unexpected content" in str(e_info.value)


def test_rhui_minor_release():
    ret = RHUIReleaseVer(context_wrap(INPUT_NORMAL_1))
    assert ret.set == '8.6'
    assert ret.major == 8
    assert ret.minor == 6


def test_rhui_minor_release_not_set():
    ret = RHUIReleaseVer(context_wrap(INPUT_NOT_SET))
    assert ret.set is None
    assert ret.major is None
    assert ret.minor is None


def test_rhui_release_2():
    ret = RHUIReleaseVer(context_wrap(INPUT_NORMAL_2))
    assert ret.set == '7Server'
    assert ret.major == 7
    assert ret.minor is None


def test_rhui_release_3():
    ret = RHUIReleaseVer(context_wrap(INPUT_NORMAL_3))
    assert ret.set == '8'
    assert ret.major == 8
    assert ret.minor is None

    ret2 = RHUIReleaseVer(context_wrap(INPUT_NG_3))
    assert ret2.set == 'ab.def'
    assert ret2.major is None
    assert ret2.minor is None


def test_rhui_release_wrong_input():
    with pytest.raises(SkipComponent) as e_info:
        RHUIReleaseVer(context_wrap(INPUT_NG_1))
    assert "Unexpected content" in str(e_info.value)


def test_doc_examples():
    env = {
            'rhui_rel': RHUISetRelease(context_wrap(INPUT_NORMAL_1)),
            'rhui_releasever': RHUIReleaseVer(context_wrap(INPUT_NORMAL_1))
          }
    failed, _ = doctest.testmod(rhui_release, globs=env)
    assert failed == 0



"""
SquidCacheLog - file ``/var/log/squid/cache.log``
=================================================
"""

from insights import parser
from insights.specs import Specs
from insights.core import LogFileOutput


@parser(Specs.squid_cache_log)
class SquidCacheLog(LogFileOutput):
    """
    Read the ``/var/log/squid/cache.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput` for more
        details.

    Sample log lines::

        2023/11/01 06:00:33 kid1|   Took 0.00 seconds (  0.00 entries/sec).
        FATAL: Couldn't start logfile helper
        Squid Cache (Version 3.5.20): Terminated abnormally.
        CPU Usage: 0.046 seconds = 0.033 user + 0.013 sys
        Maximum Resident Size: 34784 KB
        Page faults with physical i/o: 0
        2023/11/01 06:01:16 kid1| Set Current Directory to /var/spool/squid
        2023/11/01 06:01:16 kid1| Starting Squid Cache version 3.5.20 for x86_64-redhat-linux-gnu...
        2023/11/01 06:01:16 kid1| Service Name: squid

    Examples:
        >>> SquidCacheLog.last_scan('logfile_start_error', "Couldn't start logfile helper")
        >>> type(squid_cache_log)
        <class 'insights.parsers.squid.SquidCacheLog'>
        >>> error_msg = squid_cache_log.get('logfile_start_error')
        >>> error_msg.get('raw_message')
        "FATAL: Couldn't start logfile helper"
    """
    time_format = None



import doctest

from insights.parsers import pluginconf_d
from insights.tests import context_wrap


PLUGIN = """
[main]
enabled = 0
gpgcheck = 1
timeout = 120

# You can specify options per channel, e.g.:
#
#[rhel-i386-server-5]
#enabled = 1
#
#[some-unsigned-custom-channel]
#gpgcheck = 0

[test]
test_multiline_config = http://example.com/repos/test/
                        http://mirror_example.com/repos/test/
""".strip()

PLUGINPATH = '/etc/yum/plugincon.d/rhnplugin.conf'


def test_pluginconf_d_ini():
    plugin_info = pluginconf_d.PluginConfDIni(context_wrap(PLUGIN, path=PLUGINPATH))

    assert sorted(plugin_info.sections()) == sorted(['main', 'test'])
    assert 'main' in plugin_info
    assert plugin_info.get('main', 'gpgcheck') == '1'


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        pluginconf_d,
        globs={'conf': pluginconf_d.PluginConfDIni(context_wrap(PLUGIN))}
    )
    assert failed_count == 0



from insights.core.spec_factory import SpecSet, RegistryPoint


class Specs(SpecSet):
    # Client metadata specs/files
    ansible_host = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    blacklist_report = RegistryPoint(prio=-1)
    blacklisted_specs = RegistryPoint(no_obfuscate=['hostname', 'ip'], prio=-1)
    branch_info = RegistryPoint()  # https://issues.redhat.com/browse/RHIN-639
    display_name = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    egg_release = RegistryPoint()
    tags = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    version_info = RegistryPoint(no_obfuscate=['hostname', 'ip'])

    # Client App specs
    malware_detection = RegistryPoint()
    compliance = RegistryPoint()

    # Regular collection specs
    abrt_ccpp_conf = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    abrt_status_bare = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    alternatives_display_python = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    amq_broker = RegistryPoint(multi_output=True)
    audit_log = RegistryPoint(filterable=True)
    auditctl_rules = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    auditctl_status = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    auditd_conf = RegistryPoint()
    audispd_conf = RegistryPoint()
    ausearch_insights_client = RegistryPoint(prio=-1)  # The last spec to collect
    authselect_current = RegistryPoint()
    autofs_conf = RegistryPoint()
    avc_cache_threshold = RegistryPoint()
    avc_hash_stats = RegistryPoint()
    aws_instance_id_doc = RegistryPoint()
    aws_instance_id_pkcs7 = RegistryPoint()
    aws_public_ipv4_addresses = RegistryPoint()
    aws_public_hostnames = RegistryPoint()
    awx_manage_check_license = RegistryPoint()
    awx_manage_check_license_data = RegistryPoint(filterable=True)
    awx_manage_print_settings = RegistryPoint()
    azure_instance_id = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    azure_instance_plan = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    azure_instance_type = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    azure_load_balancer = RegistryPoint()
    basic_auth_insights_client = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    bdi_read_ahead_kb = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    bios_uuid = RegistryPoint()
    blkid = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    bond = RegistryPoint(multi_output=True)
    bond_dynamic_lb = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    boot_loader_entries = RegistryPoint(multi_output=True)
    bootc_status = RegistryPoint(no_obfuscate=['hostname'])
    brctl_show = RegistryPoint()
    buddyinfo = RegistryPoint()
    candlepin_broker = RegistryPoint()
    candlepin_error_log = RegistryPoint(filterable=True)
    candlepin_log = RegistryPoint(filterable=True)
    catalina_out = RegistryPoint(multi_output=True, filterable=True)
    catalina_server_log = RegistryPoint(multi_output=True, filterable=True)
    cciss = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    cdc_wdm = RegistryPoint()
    ceilometer_central_log = RegistryPoint(filterable=True)
    ceilometer_collector_log = RegistryPoint(filterable=True)
    ceilometer_compute_log = RegistryPoint(filterable=True)
    ceilometer_conf = RegistryPoint()
    ceph_conf = RegistryPoint(filterable=True)
    ceph_config_show = RegistryPoint(multi_output=True)
    ceph_df_detail = RegistryPoint()
    ceph_health_detail = RegistryPoint()
    ceph_insights = RegistryPoint()
    ceph_log = RegistryPoint(multi_output=True, filterable=True)
    ceph_osd_df = RegistryPoint()
    ceph_osd_dump = RegistryPoint()
    ceph_osd_ec_profile_get = RegistryPoint(multi_output=True)
    ceph_osd_log = RegistryPoint(multi_output=True, filterable=True)
    ceph_osd_tree = RegistryPoint()
    ceph_osd_tree_text = RegistryPoint()
    ceph_report = RegistryPoint()
    ceph_s = RegistryPoint()
    ceph_v = RegistryPoint()
    certificates_enddate = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cgroups = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    checkin_conf = RegistryPoint()
    chkconfig = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    chrony_conf = RegistryPoint()
    chronyc_sources = RegistryPoint()
    cib_xml = RegistryPoint()
    cifs_debug_data = RegistryPoint(filterable=True)
    cinder_api_log = RegistryPoint(filterable=True)
    cinder_conf = RegistryPoint()
    cinder_volume_log = RegistryPoint(filterable=True)
    cloud_cfg = RegistryPoint(filterable=True)
    cloud_cfg_filtered = RegistryPoint()
    cloud_init_custom_network = RegistryPoint()
    cloud_init_log = RegistryPoint(filterable=True)
    cluster_conf = RegistryPoint(filterable=True)
    cmdline = RegistryPoint()
    cni_podman_bridge_conf = RegistryPoint()
    cobbler_modules_conf = RegistryPoint()
    cobbler_settings = RegistryPoint()
    containers_policy = RegistryPoint()
    convert2rhel_facts = RegistryPoint()
    corosync = RegistryPoint()
    corosync_cmapctl = RegistryPoint(multi_output=True)
    corosync_conf = RegistryPoint()
    cpe = RegistryPoint()
    cpu_cores = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    cpu_siblings = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    cpu_smt_active = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cpu_smt_control = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cpu_vulns = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    cpuinfo = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cpupower_frequency_info = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cpuset_cpus = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    crictl_logs = RegistryPoint(multi_output=True, filterable=True)
    crio_conf = RegistryPoint(multi_output=True)
    cron_daily_rhsmd = RegistryPoint(filterable=True)
    cron_foreman = RegistryPoint(filterable=True)
    cron_log = RegistryPoint(filterable=True)
    crt = RegistryPoint()
    crypto_policies_bind = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    crypto_policies_config = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    crypto_policies_opensshserver = RegistryPoint()
    crypto_policies_state_current = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    cryptsetup_luksDump = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    cupsd_conf = RegistryPoint()
    cups_browsed_conf = RegistryPoint(filterable=True)
    cups_files_conf = RegistryPoint()
    cups_ppd = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    current_clocksource = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    date = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    date_utc = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    db2_database_configuration = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    db2_database_manager = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    db2ls_a_c = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    dcbtool_gc_dcb = RegistryPoint(multi_output=True)
    designate_conf = RegistryPoint(filterable=True)
    df__al = RegistryPoint()
    df__alP = RegistryPoint()
    df__li = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    dig = RegistryPoint()
    dig_dnssec = RegistryPoint()
    dig_edns = RegistryPoint()
    dig_noedns = RegistryPoint()
    dirsrv = RegistryPoint()
    dirsrv_access = RegistryPoint(multi_output=True, filterable=True)
    dirsrv_errors = RegistryPoint(multi_output=True, filterable=True)
    display_java = RegistryPoint()
    dm_mod_use_blk_mq = RegistryPoint()
    dmesg = RegistryPoint(filterable=True)
    dmesg_log = RegistryPoint(filterable=True)
    dmidecode = RegistryPoint()
    dmsetup_info = RegistryPoint()
    dmsetup_status = RegistryPoint()
    dnf_conf = RegistryPoint(filterable=True)
    dnf_module_info = RegistryPoint()
    dnf_module_list = RegistryPoint()
    dnf_modules = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    dnsmasq_config = RegistryPoint(multi_output=True)
    docker_container_inspect = RegistryPoint(multi_output=True)
    docker_host_machine_id = RegistryPoint()
    docker_info = RegistryPoint()
    docker_list_containers = RegistryPoint()
    docker_list_images = RegistryPoint()
    docker_storage = RegistryPoint()
    docker_storage_setup = RegistryPoint()
    docker_sysconfig = RegistryPoint()
    dotnet_version = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    doveconf = RegistryPoint(filterable=True)
    dracut_kdump_capture_service = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    dse_ldif = RegistryPoint(multi_output=True, filterable=True)
    du_dirs = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['hostname', 'ip'])
    dumpe2fs_h = RegistryPoint(multi_output=True)
    duplicate_machine_id = RegistryPoint(filterable=True)
    eap_json_reports = RegistryPoint(multi_output=True)
    engine_config_all = RegistryPoint()
    engine_db_query_vdsm_version = RegistryPoint()
    engine_log = RegistryPoint(filterable=True)
    etc_journald_conf = RegistryPoint()
    etc_journald_conf_d = RegistryPoint(multi_output=True)
    etc_machine_id = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    etc_udev_40_redhat_rules = RegistryPoint(filterable=True)
    etc_udev_oracle_asm_rules = RegistryPoint(multi_output=True, filterable=True)
    etcd_conf = RegistryPoint(filterable=True)
    ethtool = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_S = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_T = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_a = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_c = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_g = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_i = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ethtool_k = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    facter = RegistryPoint()
    falconctl_aid = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    falconctl_backend = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    falconctl_rfm = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    falconctl_version = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    fapolicyd_rules = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['hostname', 'ip'])
    fc_match = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    fcoeadm_i = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    findmnt_lo_propagation = RegistryPoint()
    firewall_cmd_list_all_zones = RegistryPoint()
    firewalld_conf = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    foreman_production_log = RegistryPoint(filterable=True)
    foreman_proxy_conf = RegistryPoint()
    foreman_proxy_log = RegistryPoint(filterable=True)
    foreman_rake_db_migrate_status = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    foreman_satellite_log = RegistryPoint(filterable=True)
    foreman_ssl_access_ssl_log = RegistryPoint(filterable=True)
    foreman_ssl_error_ssl_log = RegistryPoint(filterable=True)
    foreman_tasks_config = RegistryPoint(filterable=True)
    freeipa_healthcheck_log = RegistryPoint()
    fstab = RegistryPoint()
    fw_devices = RegistryPoint()
    fw_security = RegistryPoint()
    galera_cnf = RegistryPoint()
    gcp_instance_type = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    gcp_license_codes = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    gcp_network_interfaces = RegistryPoint()
    getcert_list = RegistryPoint()
    getconf_page_size = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    getenforce = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    getsebool = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    gfs2_file_system_block_size = RegistryPoint(multi_output=True)
    glance_api_log = RegistryPoint(filterable=True)
    gluster_peer_status = RegistryPoint()
    gluster_v_info = RegistryPoint()
    gluster_v_status = RegistryPoint()
    gnocchi_conf = RegistryPoint(filterable=True)
    gnocchi_metricd_log = RegistryPoint(filterable=True)
    greenboot_status = RegistryPoint(filterable=True)
    group_info = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    grub1_config_perms = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    grub2_cfg = RegistryPoint()
    grub2_efi_cfg = RegistryPoint()
    grub_conf = RegistryPoint()
    grub_config_perms = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    grub_efi_conf = RegistryPoint()
    grubby_default_index = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    grubby_default_kernel = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    grubenv = RegistryPoint()
    hammer_ping = RegistryPoint()
    hammer_task_list = RegistryPoint()
    haproxy_cfg = RegistryPoint()
    haproxy_cfg_scl = RegistryPoint()
    heat_api_log = RegistryPoint(filterable=True)
    heat_conf = RegistryPoint()
    heat_crontab = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    heat_engine_log = RegistryPoint(filterable=True)
    hostname = RegistryPoint(no_obfuscate=['ip'])
    hostname_default = RegistryPoint(no_obfuscate=['ip'])
    hostname_short = RegistryPoint(no_obfuscate=['ip'])
    hosts = RegistryPoint()
    hponcfg_g = RegistryPoint()
    httpd24_httpd_error_log = RegistryPoint(filterable=True)
    httpd_M = RegistryPoint(multi_output=True)
    httpd_V = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    httpd_access_log = RegistryPoint(filterable=True)
    httpd_cert_info_in_nss = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['ip'])
    httpd_conf = RegistryPoint(multi_output=True)
    httpd_conf_scl_httpd24 = RegistryPoint(multi_output=True)
    httpd_conf_scl_jbcs_httpd24 = RegistryPoint(multi_output=True)
    httpd_error_log = RegistryPoint(filterable=True)
    httpd_limits = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    httpd_on_nfs = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    httpd_ssl_access_log = RegistryPoint(filterable=True)
    httpd_ssl_cert_enddate = RegistryPoint(multi_output=True, no_obfuscate=['ip'])
    httpd_ssl_error_log = RegistryPoint(filterable=True)
    ibm_fw_vernum_encoded = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ibm_lparcfg = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    ifcfg = RegistryPoint(multi_output=True)
    ifcfg_static_route = RegistryPoint(multi_output=True)
    imagemagick_policy = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['ip'])
    image_builder_facts = RegistryPoint()
    init_ora = RegistryPoint()
    init_process_cgroup = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    initctl_lst = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    initscript = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    insights_client_conf = RegistryPoint(filterable=True)
    installed_rpms = RegistryPoint(no_obfuscate=['ip'])
    interrupts = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ip6tables = RegistryPoint()
    ip6tables_permanent = RegistryPoint()
    ip_addr = RegistryPoint(no_obfuscate=['hostname'])
    ip_addresses = RegistryPoint(no_obfuscate=['hostname'])
    ip_neigh_show = RegistryPoint(no_obfuscate=['hostname'])
    ip_netns_exec_namespace_lsof = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['hostname', 'ip'])
    ip_route_show_table_all = RegistryPoint(no_obfuscate=['hostname'])
    ip_s_link = RegistryPoint(no_obfuscate=['hostname'])
    ipa_default_conf = RegistryPoint()
    ipaupgrade_log = RegistryPoint(filterable=True)
    ipcs_m = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ipcs_m_p = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ipcs_s = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ipcs_s_i = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ipsec_conf = RegistryPoint(filterable=True)
    iptables = RegistryPoint()
    iptables_permanent = RegistryPoint()
    ipv4_neigh = RegistryPoint(no_obfuscate=['hostname'])
    ipv6_neigh = RegistryPoint(no_obfuscate=['hostname'])
    iris_cpf = RegistryPoint(multi_output=True)
    iris_list = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    iris_messages_log = RegistryPoint(filterable=True, multi_output=True)
    ironic_conf = RegistryPoint(filterable=True)
    ironic_inspector_log = RegistryPoint(filterable=True)
    iscsiadm_m_session = RegistryPoint()
    jbcs_httpd24_httpd_error_log = RegistryPoint(filterable=True)
    jboss_domain_server_log = RegistryPoint(multi_output=True, filterable=True)
    jboss_runtime_versions = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    jboss_standalone_main_config = RegistryPoint(multi_output=True)
    jboss_standalone_server_log = RegistryPoint(multi_output=True, filterable=True)
    jboss_version = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    journal_all = RegistryPoint(filterable=True)
    journal_header = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    journal_since_boot = RegistryPoint(filterable=True)
    katello_service_status = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    kdump_conf = RegistryPoint()
    kerberos_kdc_log = RegistryPoint(filterable=True)
    kernel_config = RegistryPoint(multi_output=True, filterable=True, no_obfuscate=['hostname', 'ip'])
    kernel_crash_kexec_post_notifiers = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    kexec_crash_loaded = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    kexec_crash_size = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    keystone_conf = RegistryPoint()
    keystone_crontab = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    keystone_log = RegistryPoint(filterable=True)
    kpatch_list = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    krb5 = RegistryPoint(multi_output=True)
    ksmstate = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ktimer_lockless = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    kubelet_conf = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    kubepods_cpu_quota = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    lastupload = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    ld_library_path_global_conf = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    leapp_migration_results = RegistryPoint()
    leapp_report = RegistryPoint()
    ld_library_path_of_user = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ldif_config = RegistryPoint(multi_output=True)
    libssh_client_config = RegistryPoint(filterable=True)
    libssh_server_config = RegistryPoint(filterable=True)
    libvirtd_log = RegistryPoint(filterable=True)
    libvirtd_qemu_log = RegistryPoint(multi_output=True, filterable=True)
    limits_conf = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    locale = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    localectl_status = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    localtime = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    login_pam_conf = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    logrotate_conf = RegistryPoint(multi_output=True)
    losetup = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lpfc_max_luns = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lpstat_p = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lpstat_protocol_printers = RegistryPoint()
    lpstat_queued_jobs_count = RegistryPoint()
    # New `ls` Specs
    ls_la = RegistryPoint()
    ls_la_dirs = RegistryPoint(filterable=True)
    ls_la_filtered = RegistryPoint(filterable=True)
    ls_la_filtered_dirs = RegistryPoint(filterable=True)
    ls_lan = RegistryPoint()
    ls_lan_dirs = RegistryPoint(filterable=True)
    ls_lan_filtered = RegistryPoint(filterable=True)
    ls_lan_filtered_dirs = RegistryPoint(filterable=True)
    ls_lanL = RegistryPoint()
    ls_lanL_dirs = RegistryPoint(filterable=True)
    ls_lanR = RegistryPoint()
    ls_lanR_dirs = RegistryPoint(filterable=True)
    ls_lanRL = RegistryPoint()
    ls_lanRL_dirs = RegistryPoint(filterable=True)
    ls_laRZ = RegistryPoint()
    ls_laRZ_dirs = RegistryPoint(filterable=True)
    ls_laZ = RegistryPoint()
    ls_laZ_dirs = RegistryPoint(filterable=True)
    # Old `ls` Specs
    ls_R_var_lib_nova_instances = RegistryPoint()
    ls_boot = RegistryPoint()
    ls_dev = RegistryPoint()
    ls_disk = RegistryPoint()
    ls_docker_volumes = RegistryPoint()
    ls_edac_mc = RegistryPoint()
    ls_etc = RegistryPoint()
    ls_etc_ssh = RegistryPoint()
    ls_ipa_idoverride_memberof = RegistryPoint()
    ls_krb5_sssd = RegistryPoint()
    ls_lib_firmware = RegistryPoint()
    ls_ocp_cni_openshift_sdn = RegistryPoint()
    ls_origin_local_volumes_pods = RegistryPoint()
    ls_osroot = RegistryPoint()
    ls_rsyslog_errorfile = RegistryPoint()
    ls_sys_firmware = RegistryPoint()
    ls_systemd_units = RegistryPoint()
    ls_tmp = RegistryPoint(filterable=True)
    ls_usr_bin = RegistryPoint(filterable=True)
    ls_usr_lib64 = RegistryPoint(filterable=True)
    ls_usr_sbin = RegistryPoint(filterable=True)
    ls_var_cache_pulp = RegistryPoint()
    ls_var_lib_mongodb = RegistryPoint()
    ls_var_lib_nova_instances = RegistryPoint()
    ls_var_lib_pcp = RegistryPoint()
    ls_var_lib_rpm = RegistryPoint()
    ls_var_lib_rsyslog = RegistryPoint()
    ls_var_log = RegistryPoint()
    ls_var_opt_mssql = RegistryPoint()
    ls_var_opt_mssql_log = RegistryPoint()
    ls_var_run = RegistryPoint()
    ls_var_spool_clientmq = RegistryPoint()
    ls_var_spool_postfix_maildrop = RegistryPoint()
    ls_var_tmp = RegistryPoint(filterable=True)
    ls_var_www = RegistryPoint()
    lsattr = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lsattr_files_or_dirs = RegistryPoint(filterable=True)
    lsblk = RegistryPoint()
    lsblk_pairs = RegistryPoint()
    lscpu = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lsinitrd = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    lsinitrd_kdump_image = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    lsinitrd_lvm_conf = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    lsmod = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lsof = RegistryPoint(filterable=True)
    lspci = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lspci_vmmkn = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lssap = RegistryPoint()
    lsscsi = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    luksmeta = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    lvdisplay = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lvm_conf = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    lvm_fullreport = RegistryPoint()
    lvm_system_devices = RegistryPoint(no_obfuscate=['ip'])
    lvmconfig = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lvs_headings = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lvs_noheadings = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    lvs_noheadings_all = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    mac_addresses = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    machine_id = RegistryPoint(no_obfuscate=['hostname', 'ip'], no_redact=True)
    manila_conf = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    mariadb_log = RegistryPoint(filterable=True)
    max_uid = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    md5chk_files = RegistryPoint(multi_output=True)
    mdadm_D = RegistryPoint()
    mdadm_E = RegistryPoint(multi_output=True)
    mdstat = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    meminfo = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    messages = RegistryPoint(filterable=True)
    metadata_json = RegistryPoint(raw=True)
    mistral_executor_log = RegistryPoint(filterable=True)
    mlx4_port = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    modinfo_filtered_modules = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    modinfo_modules = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    modprobe = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    mokutil_sbstate = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    mongod_conf = RegistryPoint(multi_output=True, filterable=True)
    mount = RegistryPoint()
    mountinfo = RegistryPoint()
    mounts = RegistryPoint()
    mpirun_version = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    mssql_api_assessment = RegistryPoint()
    mssql_conf = RegistryPoint()
    mssql_tls_cert_enddate = RegistryPoint()
    multicast_querier = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    multipath__v4__ll = RegistryPoint()
    multipath_conf = RegistryPoint()
    multipath_conf_initramfs = RegistryPoint()
    mysql_log = RegistryPoint(multi_output=True, filterable=True)
    mysqladmin_status = RegistryPoint()
    mysqladmin_vars = RegistryPoint()
    mysqld_limits = RegistryPoint()
    named_checkconf_p = RegistryPoint(filterable=True)
    named_conf = RegistryPoint(filterable=True)
    namespace = RegistryPoint()
    ndctl_list_Ni = RegistryPoint()
    netconsole = RegistryPoint()
    netstat = RegistryPoint()
    netstat_agn = RegistryPoint()
    netstat_i = RegistryPoint()
    netstat_s = RegistryPoint()
    networkmanager_conf = RegistryPoint()
    networkmanager_dispatcher_d = RegistryPoint(multi_output=True)
    neutron_conf = RegistryPoint(filterable=True)
    neutron_dhcp_agent_ini = RegistryPoint(filterable=True)
    neutron_l3_agent_ini = RegistryPoint(filterable=True)
    neutron_l3_agent_log = RegistryPoint(filterable=True)
    neutron_metadata_agent_ini = RegistryPoint(filterable=True)
    neutron_metadata_agent_log = RegistryPoint(filterable=True)
    neutron_ml2_conf = RegistryPoint(filterable=True)
    neutron_ovs_agent_log = RegistryPoint(filterable=True)
    neutron_plugin_ini = RegistryPoint()
    neutron_server_log = RegistryPoint(filterable=True)
    neutron_sriov_agent = RegistryPoint(filterable=True)
    nfnetlink_queue = RegistryPoint()
    nfs_conf = RegistryPoint()
    nfs_exports = RegistryPoint()
    nfs_exports_d = RegistryPoint(multi_output=True)
    nft_list_ruleset = RegistryPoint()
    nginx_conf = RegistryPoint(multi_output=True)
    nginx_error_log = RegistryPoint(filterable=True)
    nginx_ssl_cert_enddate = RegistryPoint(multi_output=True)
    nmap_ssh = RegistryPoint()
    nmcli_conn_show = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    nmcli_dev_show = RegistryPoint()
    nmcli_dev_show_sos = RegistryPoint(multi_output=True)
    nova_api_log = RegistryPoint(filterable=True)
    nova_compute_log = RegistryPoint(filterable=True)
    nova_conf = RegistryPoint()
    nova_crontab = RegistryPoint()
    nova_migration_uid = RegistryPoint()
    nova_uid = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    nscd_conf = RegistryPoint(filterable=True)
    nss_rhel7 = RegistryPoint()
    nsswitch_conf = RegistryPoint(filterable=True)
    ntp_conf = RegistryPoint()
    ntpq_leap = RegistryPoint()
    ntpq_pn = RegistryPoint()
    ntptime = RegistryPoint()
    numa_cpus = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    numeric_user_group_name = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    nvidia_smi_l = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    nvme_core_io_timeout = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    oc_get_bc = RegistryPoint()
    oc_get_build = RegistryPoint()
    oc_get_clusterrole_with_config = RegistryPoint()
    oc_get_clusterrolebinding_with_config = RegistryPoint()
    oc_get_configmap = RegistryPoint()
    oc_get_dc = RegistryPoint()
    oc_get_egressnetworkpolicy = RegistryPoint()
    oc_get_endpoints = RegistryPoint()
    oc_get_event = RegistryPoint()
    oc_get_node = RegistryPoint()
    oc_get_pod = RegistryPoint()
    oc_get_project = RegistryPoint()
    oc_get_pv = RegistryPoint()
    oc_get_pvc = RegistryPoint()
    oc_get_rc = RegistryPoint()
    oc_get_role = RegistryPoint()
    oc_get_rolebinding = RegistryPoint()
    oc_get_route = RegistryPoint()
    oc_get_service = RegistryPoint()
    octavia_conf = RegistryPoint(filterable=True)
    od_cpu_dma_latency = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    odbc_ini = RegistryPoint(filterable=True)
    odbcinst_ini = RegistryPoint()
    open_vm_tools_stat_raw_text_session = RegistryPoint()
    openshift_fluentd_environ = RegistryPoint(multi_output=True)
    openshift_hosts = RegistryPoint(filterable=True)
    openshift_router_environ = RegistryPoint(multi_output=True)
    openvswitch_daemon_log = RegistryPoint(filterable=True)
    openvswitch_other_config = RegistryPoint()
    openvswitch_server_log = RegistryPoint(filterable=True)
    os_release = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    osa_dispatcher_log = RegistryPoint(filterable=True)
    ose_master_config = RegistryPoint()
    ose_node_config = RegistryPoint()
    ovirt_engine_boot_log = RegistryPoint(filterable=True)
    ovirt_engine_confd = RegistryPoint(multi_output=True)
    ovirt_engine_console_log = RegistryPoint(filterable=True)
    ovirt_engine_server_log = RegistryPoint(filterable=True)
    ovirt_engine_ui_log = RegistryPoint(filterable=True)
    ovs_appctl_fdb_show_bridge = RegistryPoint(multi_output=True)
    ovs_ofctl_dump_flows = RegistryPoint(multi_output=True)
    ovs_vsctl_list_bridge = RegistryPoint()
    ovs_vsctl_show = RegistryPoint()
    ovs_vswitchd_limits = RegistryPoint()
    pacemaker_log = RegistryPoint(filterable=True)
    package_provides_command = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    pam_conf = RegistryPoint()
    parted__l = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    partitions = RegistryPoint()
    passenger_status = RegistryPoint()
    password_auth = RegistryPoint()
    pci_rport_target_disk_paths = RegistryPoint()
    pcp_metrics = RegistryPoint()
    pcp_openmetrics_log = RegistryPoint(filterable=True)
    pcp_raw_data = RegistryPoint(raw=True, multi_output=True, no_obfuscate=['hostname', 'ip'], no_redact=True)  # No Parser required
    pcs_config = RegistryPoint()
    pcs_quorum_status = RegistryPoint()
    pcs_status = RegistryPoint()
    php_ini = RegistryPoint(filterable=True)
    pidstat = RegistryPoint()
    pluginconf_d = RegistryPoint(multi_output=True)
    pmlog_summary = RegistryPoint(no_obfuscate=['hostname', 'ip'])  # TODO: Remove after 'pcp_raw_data' works
    pmlog_summary_pcp_zeroconf = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    pmrep_metrics = RegistryPoint()
    podman_list_containers = RegistryPoint()
    podman_list_images = RegistryPoint()
    postconf = RegistryPoint(filterable=True)
    postconf_builtin = RegistryPoint(filterable=True)
    postfix_master = RegistryPoint()
    postgresql_conf = RegistryPoint()
    postgresql_log = RegistryPoint(multi_output=True, filterable=True)
    proc_keys = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    proc_keyusers = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    proc_netstat = RegistryPoint()
    proc_slabinfo = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    proc_snmp_ipv4 = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    proc_snmp_ipv6 = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    proc_stat = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ps_alxwww = RegistryPoint(filterable=True)
    ps_aux = RegistryPoint(filterable=True)
    ps_auxcww = RegistryPoint()
    ps_auxww = RegistryPoint(filterable=True)
    ps_ef = RegistryPoint(filterable=True)
    ps_eo = RegistryPoint()
    ps_eo_cmd = RegistryPoint()
    pulp_worker_defaults = RegistryPoint()
    puppet_ca_cert_expire_date = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    puppet_ssl_cert_ca_pem = RegistryPoint()
    puppetserver_config = RegistryPoint(filterable=True)
    pvs_headings = RegistryPoint()
    pvs_noheadings = RegistryPoint()
    pvs_noheadings_all = RegistryPoint()
    qemu_conf = RegistryPoint()
    qemu_xml = RegistryPoint(multi_output=True)
    ql2xmaxlun = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    ql2xmqsupport = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    qpid_stat_g = RegistryPoint()
    qpid_stat_q = RegistryPoint()
    qpid_stat_u = RegistryPoint()
    qpidd_conf = RegistryPoint()
    rabbitmq_env = RegistryPoint()
    rabbitmq_logs = RegistryPoint(multi_output=True, filterable=True)
    rabbitmq_queues = RegistryPoint()
    rabbitmq_startup_err = RegistryPoint(filterable=True)
    rabbitmq_startup_log = RegistryPoint(filterable=True)
    rabbitmq_users = RegistryPoint()
    random_entropy_avail = RegistryPoint()
    rc_local = RegistryPoint()
    rdma_conf = RegistryPoint()
    readlink_e_etc_mtab = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    readlink_e_shift_cert_client = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    readlink_e_shift_cert_server = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rear_local_conf = RegistryPoint(filterable=True)
    recvq_socket_buffer = RegistryPoint()
    redhat_release = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    repquota_agnpuv = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    resolv_conf = RegistryPoint()
    rhc_conf = RegistryPoint(filterable=True)
    rhev_data_center = RegistryPoint()
    rhn_charsets = RegistryPoint()
    rhn_conf = RegistryPoint()
    rhn_entitlement_cert_xml = RegistryPoint(multi_output=True)
    rhn_hibernate_conf = RegistryPoint()
    rhn_schema_stats = RegistryPoint()
    rhn_schema_version = RegistryPoint()
    rhn_search_daemon_log = RegistryPoint(filterable=True)
    rhn_server_satellite_log = RegistryPoint(filterable=True)
    rhn_server_xmlrpc_log = RegistryPoint(filterable=True)
    rhn_taskomatic_daemon_log = RegistryPoint(filterable=False)
    rhosp_release = RegistryPoint()
    rhsm_conf = RegistryPoint(filterable=True)
    rhsm_katello_default_ca_cert = RegistryPoint()
    rhsm_log = RegistryPoint(filterable=True)
    rhsm_releasever = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rhui_releasever = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rhui_set_release = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rhv_log_collector_analyzer = RegistryPoint()
    rndc_status = RegistryPoint()
    root_crontab = RegistryPoint()
    ros_config = RegistryPoint()  # TODO: Remove after 'pcp_raw_data' works
    route = RegistryPoint()
    rpm_V_package = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    rpm_V_package_list = RegistryPoint(filterable=True)
    rpm_V_packages = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rpm_ostree_status = RegistryPoint()
    rpm_pkgs = RegistryPoint(no_obfuscate=['ip'])
    rsyslog_conf = RegistryPoint(filterable=True, multi_output=True)
    rsyslog_tls_ca_cert_enddate = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    rsyslog_tls_cert_enddate = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    samba = RegistryPoint(filterable=True)
    samba_logs = RegistryPoint(multi_output=True, filterable=True)
    sap_dev_disp = RegistryPoint(multi_output=True, filterable=True)
    sap_dev_rd = RegistryPoint(multi_output=True, filterable=True)
    sap_hana_landscape = RegistryPoint(multi_output=True)
    sap_hdb_version = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    sap_host_profile = RegistryPoint(filterable=True)
    sapcontrol_getsystemupdatelist = RegistryPoint()
    saphostctl_getcimobject_sapinstance = RegistryPoint(filterable=True)
    saphostexec_status = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    saphostexec_version = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sat5_insights_properties = RegistryPoint()
    satellite_compute_resources = RegistryPoint()
    satellite_content_hosts_count = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_core_taskreservedresource_count = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_custom_ca_chain = RegistryPoint()
    satellite_custom_hiera = RegistryPoint()
    satellite_enabled_features = RegistryPoint()
    satellite_ignore_source_rpms_repos = RegistryPoint()
    satellite_host_facts_count = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_katello_repos_with_muliple_ref = RegistryPoint()
    satellite_logs_table_size = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_missed_pulp_agent_queues = RegistryPoint()
    satellite_mongodb_storage_engine = RegistryPoint()
    satellite_non_yum_type_repos = RegistryPoint()
    satellite_provision_param_settings = RegistryPoint()
    satellite_qualified_capsules = RegistryPoint()
    satellite_qualified_katello_repos = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_revoked_cert_count = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_rhv_hosts_count = RegistryPoint()
    satellite_sca_status = RegistryPoint()
    satellite_settings = RegistryPoint()
    satellite_version_rb = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    satellite_yaml = RegistryPoint()
    sched_rt_runtime_us = RegistryPoint()
    scheduler = RegistryPoint(multi_output=True)
    scsi = RegistryPoint()
    scsi_eh_deadline = RegistryPoint(multi_output=True)
    scsi_fwver = RegistryPoint(multi_output=True)
    scsi_mod_max_report_luns = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    scsi_mod_use_blk_mq = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sctp_asc = RegistryPoint()
    sctp_eps = RegistryPoint()
    sctp_snmp = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sealert = RegistryPoint()
    secure = RegistryPoint(filterable=True)
    securetty = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    selinux_config = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    selinux_users = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    sendmail_mc = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    sendq_socket_buffer = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sestatus = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    setup_named_chroot = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    smartctl = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    smartpdc_settings = RegistryPoint(filterable=True)
    smbstatus_S = RegistryPoint()
    smbstatus_p = RegistryPoint()
    sockstat = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    softnet_stat = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    software_collections_list = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sos_conf = RegistryPoint(filterable=True)
    spamassassin_channels = RegistryPoint()
    spfile_ora = RegistryPoint(multi_output=True)
    squid_cache_log = RegistryPoint(filterable=True)
    ss = RegistryPoint()
    ssh_config = RegistryPoint(filterable=True)
    ssh_config_d = RegistryPoint(multi_output=True, filterable=True)
    ssh_foreman_config = RegistryPoint(filterable=True)
    ssh_foreman_proxy_config = RegistryPoint(filterable=True)
    sshd_config = RegistryPoint(filterable=True)
    sshd_config_d = RegistryPoint(multi_output=True, filterable=True)
    sshd_config_perms = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sshd_test_mode = RegistryPoint(filterable=True, no_redact=True)
    sssd_config = RegistryPoint()
    sssd_conf_d = RegistryPoint(multi_output=True)
    sssd_logs = RegistryPoint(multi_output=True, filterable=True)
    sys_block_queue_stable_writes = RegistryPoint(multi_output=True)
    subscription_manager_facts = RegistryPoint(filterable=True)
    subscription_manager_id = RegistryPoint(no_obfuscate=['ip'], no_redact=True)
    subscription_manager_installed_product_ids = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    subscription_manager_list_consumed = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    subscription_manager_list_installed = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    subscription_manager_release_show = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    subscription_manager_status = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sudoers = RegistryPoint(multi_output=True, filterable=True)
    swift_conf = RegistryPoint()
    swift_log = RegistryPoint(filterable=True)
    swift_object_expirer_conf = RegistryPoint()
    swift_proxy_server_conf = RegistryPoint()
    sys_fs_cgroup_memory_tasks_number = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sys_fs_cgroup_uniq_memory_swappiness = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sys_kernel_sched_features = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    sys_vmbus_class_id = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    sys_vmbus_device_id = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    sysconfig_chronyd = RegistryPoint()
    sysconfig_grub = RegistryPoint()
    sysconfig_httpd = RegistryPoint()
    sysconfig_irqbalance = RegistryPoint()
    sysconfig_kdump = RegistryPoint()
    sysconfig_kernel = RegistryPoint()
    sysconfig_libvirt_guests = RegistryPoint()
    sysconfig_memcached = RegistryPoint()
    sysconfig_mongod = RegistryPoint(multi_output=True)
    sysconfig_network = RegistryPoint()
    sysconfig_nfs = RegistryPoint()
    sysconfig_ntpd = RegistryPoint()
    sysconfig_oracleasm = RegistryPoint()
    sysconfig_pcsd = RegistryPoint()
    sysconfig_prelink = RegistryPoint()
    sysconfig_sbd = RegistryPoint()
    sysconfig_sshd = RegistryPoint()
    sysconfig_stonith = RegistryPoint()
    sysconfig_virt_who = RegistryPoint()
    sysctl = RegistryPoint()
    sysctl_conf = RegistryPoint()
    sysctl_conf_initramfs = RegistryPoint(multi_output=True)
    sysctl_d_conf_etc = RegistryPoint(multi_output=True)
    sysctl_d_conf_usr = RegistryPoint(multi_output=True)
    systemctl_cat_dnsmasq_service = RegistryPoint()
    systemctl_cat_rpcbind_socket = RegistryPoint()
    systemctl_get_default = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    systemctl_list_unit_files = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    systemctl_list_units = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    systemctl_show_all_services = RegistryPoint()
    systemctl_show_all_services_with_limited_properties = RegistryPoint()
    systemctl_show_target = RegistryPoint()
    systemctl_status_all = RegistryPoint(filterable=True)
    systemd_analyze_blame = RegistryPoint()
    systemd_docker = RegistryPoint()
    systemd_logind_conf = RegistryPoint()
    systemd_openshift_node = RegistryPoint()
    systemd_system_conf = RegistryPoint()
    systemd_system_origin_accounting = RegistryPoint()
    systemid = RegistryPoint()
    systool_b_scsi_v = RegistryPoint()
    teamdctl_config_dump = RegistryPoint(multi_output=True)
    teamdctl_state_dump = RegistryPoint(multi_output=True)
    testparm_s = RegistryPoint(filterable=True)
    testparm_v_s = RegistryPoint(filterable=True)
    thp_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    thp_use_zero_page = RegistryPoint()
    timedatectl_status = RegistryPoint()
    tmpfilesd = RegistryPoint(multi_output=True)
    tomcat_server_xml = RegistryPoint(multi_output=True)
    tomcat_vdc_fallback = RegistryPoint()
    tomcat_vdc_targeted = RegistryPoint(multi_output=True)
    tomcat_web_xml = RegistryPoint(multi_output=True)
    tty_console_active = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    tuned_adm = RegistryPoint()
    tuned_conf = RegistryPoint()
    udev_66_md_rules = RegistryPoint(filterable=True)
    udev_fc_wwpn_id_rules = RegistryPoint(filterable=True)
    uname = RegistryPoint(no_obfuscate=['ip'])
    up2date = RegistryPoint()
    up2date_log = RegistryPoint(filterable=True)
    uptime = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    users_count_map_selinux_user = RegistryPoint()
    usr_journald_conf_d = RegistryPoint(multi_output=True)
    var_qemu_xml = RegistryPoint(multi_output=True)
    vdo_status = RegistryPoint()
    vdsm_conf = RegistryPoint()
    vdsm_id = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    vdsm_import_log = RegistryPoint(multi_output=True, filterable=True)
    vdsm_log = RegistryPoint(filterable=True)
    vdsm_logger_conf = RegistryPoint()
    vgdisplay = RegistryPoint()
    vgs_headings = RegistryPoint()
    vgs_noheadings = RegistryPoint()
    vgs_noheadings_all = RegistryPoint()
    vhost_net_zero_copy_tx = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    virsh_list_all = RegistryPoint()
    virt_uuid_facts = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    virt_what = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    virt_who_conf = RegistryPoint(multi_output=True, filterable=True)
    virtlogd_conf = RegistryPoint(filterable=True)
    vma_ra_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    vmcore_dmesg = RegistryPoint(multi_output=True, filterable=True)
    vmware_tools_conf = RegistryPoint()
    vsftpd = RegistryPoint()
    vsftpd_conf = RegistryPoint(filterable=True)
    watchdog_logs = RegistryPoint(filterable=True, multi_output=True)
    wc_proc_1_mountinfo = RegistryPoint()
    x86_ibpb_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    x86_ibrs_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    x86_pti_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    x86_retp_enabled = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    xfs_info = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    xfs_quota_state = RegistryPoint(no_obfuscate=['hostname', 'ip'])
    xinetd_conf = RegistryPoint(multi_output=True)
    yum_conf = RegistryPoint(filterable=True)
    yum_list_available = RegistryPoint(no_obfuscate=['ip'])
    yum_list_installed = RegistryPoint(no_obfuscate=['ip'])
    yum_log = RegistryPoint(filterable=True, no_obfuscate=['ip'])
    yum_repolist = RegistryPoint()
    yum_repos_d = RegistryPoint(multi_output=True)
    yum_updateinfo = RegistryPoint(no_obfuscate=['ip'])
    yum_updates = RegistryPoint(no_obfuscate=['ip'])
    zdump_v = RegistryPoint()
    zipl_conf = RegistryPoint()

    # container_specs
    container_cpu_online = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    container_cpuset_cpus = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    container_dotnet_version = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    container_installed_rpms = RegistryPoint(multi_output=True, no_obfuscate=['ip'])
    container_inspect_keys = RegistryPoint(filterable=True, no_obfuscate=['hostname', 'ip'])
    container_mssql_api_assessment = RegistryPoint(multi_output=True)
    container_nginx_conf = RegistryPoint(multi_output=True)
    container_nginx_error_log = RegistryPoint(multi_output=True, filterable=True)
    container_ps_aux = RegistryPoint(multi_output=True, filterable=True)
    container_redhat_release = RegistryPoint(multi_output=True, no_obfuscate=['hostname', 'ip'])
    container_vsftpd_conf = RegistryPoint(multi_output=True, filterable=True)
    containers_inspect = RegistryPoint(no_obfuscate=['hostname', 'ip'])



"""
Satellite installer configuration files
=======================================

Parsers included in this module are:

CustomHiera - file ``/etc/foreman-installer/custom-hiera.yaml``
---------------------------------------------------------------
Parsers the file `/etc/foreman-installer/custom-hiera.yaml`

"""
from insights.core import YAMLParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.satellite_custom_hiera)
class CustomHiera(YAMLParser):
    """
    Class to parse ``/etc/foreman-installer/custom-hiera.yaml``

    Examples:
        >>> 'apache::mod::prefork::serverlimit' in custom_hiera
        True
        >>> custom_hiera['apache::mod::prefork::serverlimit']
        582
    """
    def parse_content(self, content):
        try:
            super(CustomHiera, self).parse_content(content)
        except SkipComponent:
            pass



"""
Combiners - command ``modinfo <module_name>``
=============================================

ModulesInfo
-----------
The ModulesInfo combines the collected modules info from the result of
``KernelModulesInfo``.
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.modinfo import KernelModulesInfo


@combiner([KernelModulesInfo])
class ModulesInfo(dict):
    """
    Combiner to combine the result of KernelModulesInfo which supports filter
    and the parsers which only support one single module. It refers
    ``KernelModulesInfo`` first.

    Examples:
        >>> type(modules_obj)
        <class 'insights.combiners.modinfo.ModulesInfo'>
        >>> 'i40e' in modules_obj
        True
        >>> 'bnx2x' in modules_obj.retpoline_y
        False

    Raises:
        SkipComponent: When content is empty.

    Attributes:
        retpoline_y (set): A set of names of the modules with the attribute "retpoline: Y".
        retpoline_n (set): A set of names of the modules with the attribute "retpoline: N".
    """
    def __init__(self, filtered_modules_info):
        self.retpoline_y = set()
        self.retpoline_n = set()
        if filtered_modules_info:
            self.update(filtered_modules_info)
            self.retpoline_n = filtered_modules_info.retpoline_n
            self.retpoline_y = filtered_modules_info.retpoline_y
        if not self:
            raise SkipComponent



"""
LsOsroot - command ``ls -lan /``
================================

The ``ls -lan /`` command provides information for only the ``/`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 5256
    dr-xr-xr-x.  17 0 0     271 Apr  5 18:08 .
    dr-xr-xr-x.  17 0 0     271 Apr  5 18:08 ..
    -rw-r--r--.   1 0 0       0 Feb 25  2017 1
    lrwxrwxrwx.   1 0 0       7 Feb 25  2017 bin -> usr/bin
    dr-xr-xr-x.   3 0 0    4096 Feb 25  2017 boot
    -rw-r--r--.   1 0 0 5168141 Oct 16  2017 channel-list
    drwxr-xr-x.  21 0 0    3440 Apr 12 14:46 dev
    drwxr-xr-x. 148 0 0    8192 Apr 18 09:17 etc
    drwxr-xr-x.   5 0 0      37 Jul 31  2017 home
    lrwxrwxrwx.   1 0 0       7 Feb 25  2017 lib -> usr/lib
    lrwxrwxrwx.   1 0 0       9 Feb 25  2017 lib64 -> usr/lib64
    drwxr-xr-x.   2 0 0       6 Mar 10  2016 media
    drwxr-xr-x.   2 0 0       6 Mar 10  2016 mnt
    drwxr-xr-x.   5 0 0      48 Mar 27 13:37 opt
    dr-xr-xr-x. 265 0 0       0 Apr  6 02:07 proc
    -rw-r--r--.   1 0 0  175603 Apr  5 18:08 .readahead
    dr-xr-x---.  26 0 0    4096 Apr 18 09:17 root
    drwxr-xr-x.  43 0 0    1340 Apr 18 09:17 run
    lrwxrwxrwx.   1 0 0       8 Feb 25  2017 sbin -> usr/sbin
    drwxr-xr-x.   2 0 0       6 Mar 10  2016 srv
    dr-xr-xr-x.  13 0 0       0 Apr  5 18:07 sys
    drwxrwxrwt.  40 0 0    8192 Apr 18 11:17 tmp
    drwxr-xr-x.  13 0 0     155 Feb 25  2017 usr
    drwxr-xr-x.  21 0 0    4096 Apr  6 02:07 var

Examples:

    >>> ls_osroot.listing_of("/")['tmp'] == {'group': '0', 'name': 'tmp', 'links': 40, 'perms': 'rwxrwxrwt.', 'raw_entry': 'drwxrwxrwt.  40 0 0    8192 Apr 18 11:17 tmp', 'owner': '0', 'date': 'Apr 18 11:17', 'type': 'd', 'dir': '/', 'size': 8192}
    True
    >>> ls_osroot.dir_entry("/", 'tmp')['perms']
    'rwxrwxrwt.'
"""


from .. import FileListing
from .. import parser, CommandParser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_osroot)
class LsOsroot(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlan` instead.

    Parses output of ``ls -lan /`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsOsroot, "Please use the :class:`insights.parsers.ls.LSlan` instead.", "3.5.0")
        super(LsOsroot, self).__init__(*args, **kwargs)



"""
VMwareToolsConf - file ``/etc/vmware-tools/tools.conf``
=======================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.vmware_tools_conf)
class VMwareToolsConf(IniConfigFile):
    """
    The VMware tools configuration file ``/etc/vmware-tools/tools.conf``
    is in the standard 'ini' format and is read by the IniConfigFile
    parser. ``vmtoolsd.service`` provided by ``open-vm-tools`` package is
    configured using ``/etc/vmware-tools/tools.conf``.

    Sample ``/etc/vmware-tools/tools.conf`` file::

        [guestinfo]
        disable-query-diskinfo = true

        [logging]
        log = true
        vmtoolsd.level = debug
        vmtoolsd.handler = file
        vmtoolsd.data = /tmp/vmtoolsd.log

    Examples:
        >>> type(conf)
        <class 'insights.parsers.vmware_tools_conf.VMwareToolsConf'>
        >>> list(conf.sections()) == [u'guestinfo', u'logging']
        True
        >>> conf.has_option('guestinfo', 'disable-query-diskinfo')
        True
        >>> conf.getboolean('guestinfo', 'disable-query-diskinfo')
        True
        >>> conf.get('logging', 'vmtoolsd.data')
        '/tmp/vmtoolsd.log'
    """
    pass



"""
IPA - Combiner for RHEL IdM / FreeIPA information
=================================================
"""

from insights.combiners.sssd_conf import SSSDConfAll
from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.ipa_conf import IPAConfig


@combiner(IPAConfig, SSSDConfAll, InstalledRpms)
class IPA(object):
    """Combiner for IPA, SSSD, and installed RPMs

    Provides additional information, e.g. whether the host is an IPA server.
    """

    def __init__(self, ipa_conf, sssd_conf, rpms):
        self._ipa_conf = ipa_conf
        self._sssd_conf = sssd_conf

        self._is_client = None
        self._is_server = None
        self._ipa_domains = None

        self._check_installed_packages(rpms)

    def _check_installed_packages(self, rpms):
        # IPA is relying on SSSD which will be installed on both client and server
        if rpms.get_max("sssd") is None:
            raise SkipComponent("sssd package is not installed")

    @property
    def ipa_conf(self):
        """Get IPAConfig object"""
        return self._ipa_conf

    @property
    def sssd_conf(self):
        """Get SSSD_Config object"""
        return self._sssd_conf

    @property
    def sssd_ipa_domains(self):
        """Get all SSSD domains where id_provider is set to ipa"""
        if self._ipa_domains is None:
            self._ipa_domains = []
            for domain in self.sssd_conf.enabled_domains:
                id_provider = self.sssd_conf.domain_get(domain, "id_provider")
                if id_provider == "ipa":
                    self._ipa_domains.append(domain)

        return self._ipa_domains

    @property
    def is_client(self):
        """Is the host an IPA client?"""
        # Check if there is at least one IPA domain in SSSD.
        if self._is_client is None:
            self._is_client = len(self.sssd_ipa_domains) > 0

        return self._is_client

    @property
    def is_server(self):
        """Is the host an IPA server?"""
        if self._is_server is None:
            for domain in self.sssd_ipa_domains:
                self._is_server = self.sssd_conf.domain_getboolean(
                    domain, "ipa_server_mode", False
                )

                # Break if there is at least one IPA domain in server mode
                if self._is_server:
                    break

        return self._is_server



"""
LsUsrBin - command ``ls -lan /usr/bin``
=======================================

The ``ls -lan /usr/bin`` command provides information for the listing of the
``/usr/bin`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

For ls_usr_bin, it may collect a lot of files or directories that may not be
necessary, so a default filter `add_filter(Specs.ls_usr_bin, "total")` has been
added in this parser.

If addtional file or directory need to be collected by this parser, please
add related filter to corresponding code.

Sample added filter:

    >>> add_filter(Specs.ls_usr_bin, "python")
    >>> add_filter(Specs.ls_usr_bin, "virt")

Sample directory list collected::

    total 41472
    lrwxrwxrwx.  1 0  0        7 Oct 22  2019 python -> python2
    -rwxr-xr-x.  1 0  0     2558 Apr 10  2019 python-argcomplete-check-easy-install-script
    -rwxr-xr-x.  1 0  0      318 Apr 10  2019 python-argcomplete-tcsh
    lrwxrwxrwx.  1 0  0       14 Oct 22  2019 python-config -> python2-config
    lrwxrwxrwx.  1 0  0        9 Oct 22  2019 python2 -> python2.7

Examples:

    >>> "accessdb" in ls_usr_bin
    False
    >>> "/usr/bin" in ls_usr_bin
    True
    >>> ls_usr_bin.dir_entry('/usr/bin', 'python-argcomplete-tcsh')['type']
    '-'
    >>> ls_usr_bin.dir_entry('/usr/bin', 'python2')['type']
    'l'
"""

from insights.core.filters import add_filter
from insights.specs import Specs
from insights.util import deprecated

from .. import CommandParser, parser
from .. import FileListing


add_filter(Specs.ls_usr_bin, "total")


@parser(Specs.ls_usr_bin)
class LsUsrBin(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanFiltered` instead.

    Parses output of ``ls -lan /usr/bin`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsUsrBin, "Please use the :class:`insights.parsers.ls.LSlanFiltered` instead.", "3.5.0")
        super(LsUsrBin, self).__init__(*args, **kwargs)
